/*
  vector_numeric_ordered_operators.hpp

  Copyright (C) 2016 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _VECTOR_NUMERIC_ORDERED_OPERATORS_HPP_
#define _VECTOR_NUMERIC_ORDERED_OPERATORS_HPP_

template <typename T, typename U>
inline
bool ordered_operator_less(const std::vector<T> &a, const std::vector<U> &b)
{
#if 0
  return std::lexicographical_compare(&a[0], &a[a.size()], &b[0], &b[b.size()]);
#else
  for (size_t i=0; i<a.size() && i<b.size(); ++i) {
    if (a[i] < b[i])
      return true;
    if (b[i] < a[i])
      return false;
  }
  return a.size() < b.size();
#endif
}
template <typename T, typename U>
inline
bool ordered_operator_less_equal(const std::vector<T> &a, const std::vector<U> &b)
{
  return !ordered_operator_less(b, a);
}
template <typename T, typename U>
inline
bool ordered_operator_greater(const std::vector<T> &a, const std::vector<U> &b)
{
  return ordered_operator_less(b, a);
}
template <typename T, typename U>
inline
bool ordered_operator_greater_equal(const std::vector<T> &a, const std::vector<U> &b)
{
  return ordered_operator_less_equal(b, a);
}
template <typename T, typename U>
inline
bool ordered_operator_equal_to(const std::vector<T> &a, const std::vector<U> &b)
{
#if 0
  return !ordered_operator_less(a, b) && !ordered_operator_less(b, a);
#elif !0
  return a.size() == b.size() && std::equal(&a[0], &a[a.size()], &b[0]);
#else
  if (!(a.size() == b.size())) return false;
  for (size_t i=0; i<a.size(); ++i)
    if (!(a[i] == b[i]))
      return false;
  return true;
#endif
}
template <typename T, typename U>
inline
bool ordered_operator_not_equal_to(const std::vector<T> &a, const std::vector<U> &b)
{
#if 0
  return ordered_operator_less(a, b) || ordered_operator_less(b, a);
#elif !0
  return !ordered_operator_equal_to(a, b);
#else
  if (a.size() != b.size()) return true;
  for (size_t i=0; i<a.size(); ++i)
    if (a[i] != b[i])
      return true;
  return false;
#endif
}

#endif
